@@ -1,5 +1,6 @@
 import json
 from .future import DSSFuture
+from ..utils import CallableStr
 
 class DSSAPIDeployer(object):
     """
@@ -10,7 +11,7 @@ class DSSAPIDeployer(object):
     def __init__(self, client):
         self.client = client
 
-    def list_deployments(self, as_objects = True):
+    def list_deployments(self, as_objects=True):
         """
         Lists deployments on the API Deployer
 
@@ -55,7 +56,16 @@ class DSSAPIDeployer(object):
         self.client._perform_json("POST", "/api-deployer/deployments", body=settings)
         return self.get_deployment(deployment_id)
 
-    def list_infras(self, as_objects = True):
+    def list_stages(self):
+        """
+        Lists infrastructure stages of the API Deployer
+
+        :rtype: list of dict. Each dict contains a field "id" for the stage identifier and "desc" for its description.
+        :rtype: list
+        """
+        return self.client._perform_json("GET", "/api-deployer/stages")
+
+    def list_infras(self, as_objects=True):
         """
         Lists deployment infrastructures on the API Deployer
 
@@ -97,7 +107,7 @@ class DSSAPIDeployer(object):
         """
         return DSSAPIDeployerInfra(self.client, infra_id)
 
-    def list_services(self, as_objects = True):
+    def list_services(self, as_objects=True):
         """
         Lists API services on the API Deployer
 
@@ -121,7 +131,7 @@ class DSSAPIDeployer(object):
         :rtype: :class:`DSSAPIDeployerService`
         """
         settings = {
-            "serviceId" : service_id
+            "publishedServiceId" : service_id
         }
         self.client._perform_json("POST", "/api-deployer/services", body=settings)
         return self.get_service(service_id)
@@ -142,7 +152,7 @@ class DSSAPIDeployer(object):
 
 class DSSAPIDeployerInfra(object):
     """
-    A Deployment infrastructure on the API Deployer
+    An API Deployer infrastructure.
 
     Do not create this directly, use :meth:`~dataikuapi.dss.apideployer.DSSAPIDeployer.get_infra`
     """
@@ -150,8 +160,19 @@ class DSSAPIDeployerInfra(object):
         self.client = client
         self.infra_id = infra_id
 
+    @property
     def id(self):
-        return self.infra_id
+        return CallableStr(self.infra_id)
+
+    def get_status(self):
+        """
+        Returns status information about this infrastructure
+
+        :rtype: :class:`dataikuapi.dss.apideployer.DSSAPIDeployerInfraStatus`
+        """
+        light = self.client._perform_json("GET", "/api-deployer/infras/%s" % (self.infra_id))
+
+        return DSSAPIDeployerInfraStatus(self.client, self.infra_id, light)
 
     def get_settings(self):
         """
@@ -176,7 +197,8 @@ class DSSAPIDeployerInfra(object):
 
 
 class DSSAPIDeployerInfraSettings(object):
-    """The settings of an API Deployer Infra. 
+    """
+    The settings of an API Deployer infrastructure
 
     Do not create this directly, use :meth:`~dataikuapi.dss.apideployer.DSSAPIDeployerInfra.get_settings`
     """
@@ -228,6 +250,35 @@ class DSSAPIDeployerInfraSettings(object):
                 body = self.settings)
 
 
+class DSSAPIDeployerInfraStatus(object):
+    """
+    The status of an API Deployer infrastructure.
+
+    Do not create this directly, use :meth:`~dataikuapi.dss.apideployer.DSSAPIDeployerInfra.get_status`
+    """
+    def __init__(self, client, infra_id, light_status):
+        self.client = client
+        self.infra_id = infra_id
+        self.light_status = light_status
+
+    def get_deployments(self):
+        """
+        Returns the deployments that are deployed on this infrastructure
+
+        :returns: a list of deployments
+        :rtype: list of :class:`dataikuapi.dss.apideployer.DSSAPIDeployerDeployment`
+        """
+        return [DSSAPIDeployerDeployment(self.client, deployment["id"]) for deployment in self.light_status["deployments"]]
+
+    def get_raw(self):
+        """
+        Gets the raw status information. This returns a dictionary with various information about the infrastructure
+
+        :rtype: dict
+        """
+        return self.light_status
+
+
 ###############################################
 # Deployments
 ###############################################
@@ -242,11 +293,13 @@ class DSSAPIDeployerDeployment(object):
         self.client = client
         self.deployment_id = deployment_id
 
+    @property
     def id(self):
-        return self.deployment_id
+        return CallableStr(self.deployment_id)
 
     def get_status(self):
-        """Returns status information about this deployment
+        """
+        Returns status information about this deployment
 
         :rtype: dataikuapi.dss.apideployer.DSSAPIDeployerDeploymentStatus
         """
@@ -292,7 +345,8 @@ class DSSAPIDeployerDeployment(object):
 
 
 class DSSAPIDeployerDeploymentSettings(object):
-    """The settings of an API Deployer deployment. 
+    """
+    The settings of an API Deployer deployment. 
 
     Do not create this directly, use :meth:`~dataikuapi.dss.apideployer.DSSAPIDeployerDeployment.get_settings`
     """
@@ -311,7 +365,9 @@ class DSSAPIDeployerDeploymentSettings(object):
         return self.settings
 
     def set_enabled(self, enabled):
-        """Enables or disables this deployment"""
+        """
+        Enables or disables this deployment
+        """
         self.settings["enabled"] = enabled
 
     def set_single_version(self, version):
@@ -326,13 +382,17 @@ class DSSAPIDeployerDeploymentSettings(object):
         }
         
     def save(self):
-        """Saves back these settings to the deployment"""
+        """
+        Saves back these settings to the deployment
+        """
         self.client._perform_empty(
                 "PUT", "/api-deployer/deployments/%s/settings" % (self.deployment_id),
                 body = self.settings)
 
+
 class DSSAPIDeployerDeploymentStatus(object):
-    """The status of an API Deployer deployment. 
+    """
+    The status of an API Deployer deployment. 
 
     Do not create this directly, use :meth:`~dataikuapi.dss.apideployer.DSSAPIDeployerDeployment.get_status`
     """
@@ -354,12 +414,15 @@ class DSSAPIDeployerDeploymentStatus(object):
     def get_heavy(self):
         """
         Gets the 'heavy' (full) status. This returns a dictionary with various information about the deployment
+
         :rtype: dict
         """
         return self.heavy_status
 
     def get_service_urls(self):
-        """Returns service-level URLs for this deployment (ie without the enpdoint-specific suffix)"""
+        """
+        Returns service-level URLs for this deployment (ie without the enpdoint-specific suffix)
+        """
 
         if "deployedServiceId" in self.light_status["deploymentBasicInfo"]:
             service_id = self.light_status["deploymentBasicInfo"]["deployedServiceId"]
@@ -374,7 +437,8 @@ class DSSAPIDeployerDeploymentStatus(object):
             raise ValueError("PublicURL not available for this deployment. It might still be initializing")
 
     def get_health(self):
-        """Returns the health of this deployment as a string
+        """
+        Returns the health of this deployment as a string
 
         :returns: HEALTHY if the deployment is working properly, various other status otherwise
         :rtype: string
@@ -386,7 +450,6 @@ class DSSAPIDeployerDeploymentStatus(object):
         return self.heavy_status["healthMessages"]
 
 
-
 ###############################################
 # Published Service
 ###############################################
@@ -401,8 +464,9 @@ class DSSAPIDeployerService(object):
         self.client = client
         self.service_id = service_id
 
+    @property
     def id(self):
-        return self.service_id
+        return CallableStr(self.service_id)
 
     def get_status(self):
         """
@@ -414,16 +478,15 @@ class DSSAPIDeployerService(object):
         light = self.client._perform_json("GET", "/api-deployer/services/%s" % (self.service_id))
         return DSSAPIDeployerServiceStatus(self.client, self.service_id, light)
 
-    def import_version(self, version_id, fp):
+    def import_version(self, fp):
         """
         Imports a new version for an API service from a file-like object pointing 
         to a version package Zip file
 
-        :param string version_id: identifier of the new version
         :param string fp: A file-like object pointing to a version package Zip file
         """
         return self.client._perform_empty("POST",
-                "/api-deployer/services/%s/packages/%s" % (self.service_id, version_id), files={"file":fp})
+                "/api-deployer/services/%s/versions" % (self.service_id), files={"file":fp})
 
     def get_settings(self):
         """
@@ -440,9 +503,28 @@ class DSSAPIDeployerService(object):
 
         return DSSAPIDeployerServiceSettings(self.client, self.service_id, settings)
 
+    def delete_version(self, version):
+        """
+        Deletes a version from this service
+
+        :param string version: The version to delete
+        """
+        self.client._perform_empty(
+            "DELETE", "/api-deployer/services/%s/versions/%s" % (self.service_id, version))
+
+    def delete(self):
+        """
+        Deletes this service
+
+        You may only delete a service if it has no deployments on it anymore.
+        """
+        return self.client._perform_empty(
+            "DELETE", "/api-deployer/services/%s" % (self.service_id))
+
 
 class DSSAPIDeployerServiceSettings(object):
-    """The settings of an API Deployer Service. 
+    """
+    The settings of an API Deployer Service. 
 
     Do not create this directly, use :meth:`~dataikuapi.dss.apideployer.DSSAPIDeployerService.get_settings`
     """
@@ -461,13 +543,17 @@ class DSSAPIDeployerServiceSettings(object):
         return self.settings
 
     def save(self):
-        """Saves back these settings to the API service"""
+        """
+        Saves back these settings to the API service
+        """
         self.client._perform_empty(
                 "PUT", "/api-deployer/services/%s/settings" % (self.service_id),
                 body = self.settings)
 
+
 class DSSAPIDeployerServiceStatus(object):
-    """The status of an API Deployer Service. 
+    """
+    The status of an API Deployer Service. 
 
     Do not create this directly, use :meth:`~dataikuapi.dss.apideployer.DSSAPIDeployerService.get_status`
     """
@@ -476,6 +562,20 @@ class DSSAPIDeployerServiceStatus(object):
         self.service_id = service_id
         self.light_status = light_status
 
+    def get_deployments(self, infra_id=None):
+        """
+        Returns the deployments that have been created from this published project
+
+        :param str infra_id: Identifier of an infra, allows to only keep in the returned list the deployments on this infra.
+        If not set, the list contains all the deployments using this published project, across every infra of the Project Deployer.
+
+        :returns: a list of deployments
+        :rtype: list of :class:`dataikuapi.dss.apideployer.DSSAPIDeployerDeployment`
+        """
+        if infra_id is None:
+            return [DSSAPIDeployerDeployment(self.client, deployment["id"]) for deployment in self.light_status["deployments"]]
+        return [DSSAPIDeployerDeployment(self.client, deployment["id"]) for deployment in self.light_status["deployments"] if infra_id == deployment["infraId"]]
+
     def get_versions(self):
         """
         Returns the versions of this service that have been published on the API Service
@@ -490,6 +590,7 @@ class DSSAPIDeployerServiceStatus(object):
     def get_raw(self):
         """
         Gets the raw status information. This returns a dictionary with various information about the service,
+
         :rtype: dict
         """
         return self.light_status
