@@ -1,25 +1,28 @@
 import json
+
 from requests import Session
 from requests import exceptions
 from requests.auth import HTTPBasicAuth
 
+from dataikuapi.dss.notebook import DSSNotebook
 from .dss.future import DSSFuture
 from .dss.projectfolder import DSSProjectFolder
 from .dss.project import DSSProject
+from .dss.app import DSSApp
 from .dss.plugin import DSSPlugin
-from .dss.admin import DSSUser, DSSGroup, DSSConnection, DSSGeneralSettings, DSSCodeEnv, DSSGlobalApiKey, DSSCluster
+from .dss.admin import DSSUser, DSSOwnUser, DSSGroup, DSSConnection, DSSGeneralSettings, DSSCodeEnv, DSSGlobalApiKey, DSSCluster
 from .dss.meaning import DSSMeaning
 from .dss.sqlquery import DSSSQLQuery
-from .dss.notebook import DSSNotebook
 from .dss.discussion import DSSObjectDiscussions
 from .dss.apideployer import DSSAPIDeployer
+from .dss.projectdeployer import DSSProjectDeployer
 import os.path as osp
 from .utils import DataikuException
 
 class DSSClient(object):
     """Entry point for the DSS API client"""
 
-    def __init__(self, host, api_key=None, internal_ticket = None):
+    def __init__(self, host, api_key=None, internal_ticket = None, extra_headers = None):
         """
         Instantiate a new DSS API client on the given host with the given API key.
 
@@ -38,8 +41,10 @@ class DSSClient(object):
             self._session.headers.update({"X-DKU-APITicket" : self.internal_ticket})
         else:
             raise ValueError("API Key is required")
-            
-            
+
+        if extra_headers is not None:
+            self._session.headers.update(extra_headers)
+
     ########################################################
     # Futures
     ########################################################
@@ -86,7 +91,7 @@ class DSSClient(object):
     ########################################################
     # Notebooks
     ########################################################
-            
+
     def list_running_notebooks(self, as_objects=True):
         """
         List the currently-running Jupyter notebooks
@@ -96,11 +101,11 @@ class DSSClient(object):
         :return: list of notebooks. if as_objects is True, each entry in the list is a :class:`dataikuapi.dss.notebook.DSSNotebook`. Else, each item in the list is a dict which contains at least a "name" field.
         :rtype: list of :class:`dataikuapi.dss.notebook.DSSNotebook` or list of dict
         """
-        list = self._perform_json("GET", "/admin/notebooks/")
+        notebook_list = self._perform_json("GET", "/admin/notebooks/")
         if as_objects:
-            return [DSSNotebook(self, notebook['projectKey'], notebook['name'], notebook) for notebook in list]
+            return [DSSNotebook(self, notebook['projectKey'], notebook['name'], notebook) for notebook in notebook_list]
         else:
-            return list
+            return notebook_list
 
     ########################################################
     # Project folders
@@ -111,8 +116,7 @@ class DSSClient(object):
 
         :returns: A :class:`dataikuapi.dss.projectfolder.DSSProjectFolder`to interact with this project folder
         """
-        project_folder_id = self._perform_json("GET", "/project-folders/")["id"]
-        return DSSProjectFolder(self, project_folder_id)
+        return self.get_project_folder("ROOT")
 
     def get_project_folder(self, project_folder_id):
         """
@@ -121,7 +125,8 @@ class DSSClient(object):
         :param str project_folder_id: the project folder ID of the desired project folder
         :returns: A :class:`dataikuapi.dss.projectfolder.DSSProjectFolder`to interact with this project folder
         """
-        return DSSProjectFolder(self, project_folder_id)
+        data = self._perform_json("GET", "/project-folders/%s" % project_folder_id)
+        return DSSProjectFolder(self, data)
 
     ########################################################
     # Projects
@@ -154,6 +159,13 @@ class DSSClient(object):
         """
         return DSSProject(self, project_key)
 
+    def get_default_project(self):
+        """
+        Get a handle to the current default project, if available (i.e. if dataiku.default_project_key() is valid)
+        """
+        import dataiku
+        return DSSProject(self, dataiku.default_project_key())
+
     def create_project(self, project_key, name, owner, description=None, settings=None, project_folder_id=None):
         """
         Creates a new project, and return a project handle to interact with it.
@@ -182,6 +194,28 @@ class DSSClient(object):
                }, params=params)
         return DSSProject(self, project_key)
 
+    ########################################################
+    # Apps
+    ########################################################
+
+    def list_apps(self):
+        """
+        List the apps
+
+        :returns: a list of apps, each as a dict. Each dict contains at least a 'appId' field
+        :rtype: list of dicts
+        """
+        return self._perform_json("GET", "/apps/")
+
+    def get_app(self, app_id):
+        """
+        Get a handle to interact with a specific app.
+
+        :param str app_id: the id of the desired app
+        :returns: A :class:`dataikuapi.dss.app.DSSApp` to interact with this project
+        """
+        return DSSApp(self, app_id)
+
     ########################################################
     # Plugins
     ########################################################
@@ -194,6 +228,31 @@ class DSSClient(object):
         """
         return self._perform_json("GET", "/plugins/")
 
+    def download_plugin_stream(self, plugin_id):
+        """
+        Download a development plugin, as a binary stream
+        :param str plugin_id: identifier of the plugin to download
+
+        :param plugin_id:
+        :return: the binary stream
+        """
+        return self._perform_raw("GET", "/plugins/%s/download" % plugin_id)
+
+    def download_plugin_to_file(self, plugin_id, path):
+        """
+        Download a development plugin to a file
+
+        :param str plugin_id: identifier of the plugin to download
+        :param str path: the path where to download the plugin
+        :return: None
+        """
+        stream = self.download_plugin_stream(plugin_id)
+        with open(path, 'wb') as f:
+            for chunk in stream.iter_content(chunk_size=10000):
+                if chunk:
+                    f.write(chunk)
+                    f.flush()
+
     def install_plugin_from_archive(self, fp):
         """
         Install a plugin from a plugin archive (as a file object)
@@ -213,7 +272,6 @@ class DSSClient(object):
         f = self._perform_json("POST", "/plugins/actions/installFromStore", body={
             "pluginId": plugin_id
         })
-        print(f)
         return DSSFuture(self, f["jobId"])
 
     def install_plugin_from_git(self, repository_url, checkout = "master", subpath=None):
@@ -245,7 +303,7 @@ class DSSClient(object):
     # SQL queries
     ########################################################
 
-    def sql_query(self, query, connection=None, database=None, dataset_full_name=None, pre_queries=None, post_queries=None, type='sql', extra_conf=None, script_steps=None, script_input_schema=None, script_output_schema=None, script_report_location=None, read_timestamp_without_timezone_as_string=True, read_date_as_string=False):
+    def sql_query(self, query, connection=None, database=None, dataset_full_name=None, pre_queries=None, post_queries=None, type='sql', extra_conf=None, script_steps=None, script_input_schema=None, script_output_schema=None, script_report_location=None, read_timestamp_without_timezone_as_string=True, read_date_as_string=False, project_key=None):
         """
         Initiate a SQL, Hive or Impala query and get a handle to retrieve the results of the query.
         Internally, the query is run by DSS. The  database to run the query on is specified either by 
@@ -259,12 +317,13 @@ class DSSClient(object):
         :param list pre_queries: (optional) array of queries to run before the query
         :param list post_queries: (optional) array of queries to run after the query
         :param str type: the type of query : either 'sql', 'hive' or 'impala'
-        
+        :param str project_key: The project_key on which the query should be run (especially useful for user isolation/impersonation scenario)
+
         :returns: A :class:`dataikuapi.dss.sqlquery.DSSSQLQuery` query handle
         """
         if extra_conf is None:
             extra_conf = {}
-        return DSSSQLQuery(self, query, connection, database, dataset_full_name, pre_queries, post_queries, type, extra_conf, script_steps, script_input_schema, script_output_schema, script_report_location, read_timestamp_without_timezone_as_string, read_date_as_string)
+        return DSSSQLQuery(self, query, connection, database, dataset_full_name, pre_queries, post_queries, type, extra_conf, script_steps, script_input_schema, script_output_schema, script_report_location, read_timestamp_without_timezone_as_string, read_date_as_string, project_key)
 
     ########################################################
     # Users
@@ -320,6 +379,9 @@ class DSSClient(object):
                })
         return DSSUser(self, login)
 
+    def get_own_user(self):
+        return DSSOwnUser(self)
+
     ########################################################
     # Groups
     ########################################################
@@ -786,6 +848,17 @@ class DSSClient(object):
         """
         return DSSAPIDeployer(self)
 
+    ########################################################
+    # Project Deployer
+    ########################################################
+
+    def get_projectdeployer(self):
+        """Gets a handle to work with the Project Deployer
+
+        :rtype: :class:`~dataikuapi.dss.projectdeployer.DSSProjectDeployer`
+        """
+        return DSSProjectDeployer(self)
+
     ########################################################
     # Data Catalog
     ########################################################
@@ -859,6 +932,21 @@ class DSSClient(object):
         """
         return self._perform_json("POST", "/auth/info-from-browser-headers",
                 params={"withSecrets": with_secrets}, body=headers_dict)
+        
+    def get_ticket_from_browser_headers(self, headers_dict):
+         """
+         Returns a ticket for the DSS user authenticated by the dictionary of
+         HTTP headers provided in headers_dict.
+
+         This is only used in webapp backends
+
+         This method returns a ticket to use as a X-DKU-APITicket header
+
+         :param: headers_dict dict: Dictionary of HTTP headers
+         :returns: a string
+         :rtype: string
+         """
+         return self._perform_json("POST", "/auth/ticket-from-browser-headers", body=headers_dict)
 
     def create_personal_api_key(self, label):
         """
@@ -873,6 +961,24 @@ class DSSClient(object):
         return self._perform_json("POST", "/auth/personal-api-keys",
                 params={"label": label})
 
+    ########################################################
+    # Container execution
+    ########################################################
+
+    def push_base_images(self):
+        """
+        Push base images for Kubernetes container-execution and Spark-on-Kubernetes
+        """
+        resp = self._perform_json("POST", "/admin/container-exec/actions/push-base-images")
+        return DSSFuture.from_resp(self, resp)
+
+    def apply_kubernetes_namespaces_policies(self):
+        """
+        Apply Kubernetes namespaces policies defined in the general settings
+        """
+        resp = self._perform_json("POST", "/admin/container-exec/actions/apply-kubernetes-policies")
+        return DSSFuture.from_resp(self, resp)
+
     ########################################################
     # Licensing
     ########################################################
@@ -885,6 +991,17 @@ class DSSClient(object):
         """
         return self._perform_json("GET", "/admin/licensing/status")
 
+    def set_license(self, license):
+        """
+        Sets a new licence for DSS
+
+        :param license: license (content of license file)
+        :return: None
+        """
+        self._perform_empty(
+            "POST", "/admin/licensing/license", body=json.loads(license))
+
+
     ########################################################
     # Internal Request handling
     ########################################################
@@ -948,7 +1065,6 @@ class DSSClient(object):
         """
         return DSSObjectDiscussions(self, project_key, object_type, object_id)
 
-
 class TemporaryImportHandle(object):
     def __init__(self, client, import_id):
         self.client = client
